import 'package:equatable/equatable.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/communities/domain/models/community_model.dart';

class CommunitiesPaginationModel extends Equatable {
  const CommunitiesPaginationModel({
    required this.communities,
    required this.page,
    required this.size,
    required this.hasNext,
    required this.totalItems,
    required this.totalPages,
  });

  final List<CommunityModel> communities;
  final int page;
  final int size;
  final bool hasNext;
  final int totalItems;
  final int totalPages;

  const CommunitiesPaginationModel.empty()
      : communities = const [],
        page = 1,
        size = 25,
        hasNext = false,
        totalItems = 0,
        totalPages = 0;

  factory CommunitiesPaginationModel.fromJson(Map<String, dynamic> json) {
    return CommunitiesPaginationModel(
      communities: (json['data'] as List<dynamic>? ?? [])
          .map((e) => CommunityModel.fromJson(e as Map<String, dynamic>))
          .toList(),
      page: json['page'] as int? ?? 1,
      size: json['size'] as int? ?? 25,
      hasNext: json['hasNext'] as bool? ?? false,
      totalItems: json['totalItem'] as int? ?? 0,
      totalPages: json['totalPage'] as int? ?? 0,
    );
  }

  CommunitiesPaginationModel copyWith({
    List<CommunityModel>? communities,
    int? page,
    int? size,
    bool? hasNext,
    int? totalItems,
    int? totalPages,
  }) {
    return CommunitiesPaginationModel(
      communities: communities ?? this.communities,
      page: page ?? this.page,
      size: size ?? this.size,
      hasNext: hasNext ?? this.hasNext,
      totalItems: totalItems ?? this.totalItems,
      totalPages: totalPages ?? this.totalPages,
    );
  }

  @override
  List<Object?> get props => [
        communities,
        page,
        size,
        hasNext,
        totalItems,
        totalPages,
      ];
}
